@@ -40,7 +40,7 @@ sys.path.append('.')
 import utils
 
 class Scriber(object):
-    def __init__(self):
+    def __init__(self, filtered=[]):
         pass
 
     def p(self, obj):
@@ -81,14 +81,16 @@ class Watcher(object):
 
 
 class Runner(object):
-    def __init__(self):
+    def __init__(self, logging):
         self.show_line_num = True
-        self.save_logs = True
+        self.save_logs = logging
+
         self.initialized = False
         # p for print, d for distinguish
         self.api_calls = ['p', 'watch', 'iterscribe', 'd', 'Scriber']
-        self.imports = ['re', 'pprint']
+        self.imports = ['re', 'pprint', 'datetime']
         self.desugared_lines = []
+        self.filtered_labels = []
         self.watcher = Watcher()
 
     def gen_line_mapping(self, program_file):
@@ -155,23 +157,29 @@ class Runner(object):
                     first_call_indentation != "" and
                     len(line_content) > 2 and
                     len(indentation) < len(first_call_indentation)):
-                closing_line = (first_call_indentation +
+                closing_line = (utils.get_end(first_call_indentation) +
+                                first_call_indentation +
                                 "pyscribe_log.close()\n")
                 closing_line_added = True
                 self.desugared_lines.append(closing_line)
-            if "Scriber()" in line_content:  # Line matches initial call
-                if self.save_logs:
+            if "pyscribe" in line_content:
+                if self.save_logs and "Scriber(" in line_content:
                     self.initialized = True
+                    timestamp = utils.get_timestamp(indentation)
                     desugared_line = (indentation +
                                      "pyscribe_log = open('pyscribe_logs.txt', 'w')\n")
-                    self.desugared_lines.append(desugared_line)
+                    self.desugared_lines.append(desugared_line + timestamp)
                     first_call_indentation = indentation
-                self.desugared_lines.append(line_content)
+                if "Scriber(" in line_content:
+                    self.filtered_labels = utils.get_filtered_labels(line_content, program_ast)
+                self.desugared_lines.append('#' + line_content)
             elif line_content in line_mapping.values():  # Line matches an API call
-                self.desugared_lines.append(self.desugar_line(line_content[:-1],
-                                            line_num,
-                                            program_file,
-                                            program_ast))  # don't want to include \n
+                desugared_line = self.desugar_line(line_content[:-1],
+                                                   line_num,
+                                                   program_file,
+                                                   program_ast)  # don't want to include \n
+                if desugared_line:
+                    self.desugared_lines.append(desugared_line)
             elif (self.watcher.num_watched() > 0 and  # Line matches watched variable change
                   line_num in self.watcher.new_line_nums()):
                 self.desugared_lines.append(line_content)
@@ -184,11 +192,13 @@ class Runner(object):
             else:
                 self.desugared_lines.append(line_content)
         if not closing_line_added and self.save_logs and self.initialized:
-            self.desugared_lines.append("pyscribe_log.close()\n")
+            self.desugared_lines.append(utils.get_end('') + "pyscribe_log.close()\n")
         program.close()
+
         for line in self.desugared_lines:
             desugared_copy.write(line)
         desugared_copy.close()
+
         return desugared_copy_name
 
     def from_line(self, line_num):
@@ -223,6 +233,9 @@ class Runner(object):
         else:
             desugared_line = ""
 
+        if not desugared_line:
+            return ""
+
         action, ending = self.action_and_ending(line_num)
         output = action + self.from_line(line_num) + desugared_line + ending
 
@@ -252,13 +265,20 @@ class Runner(object):
                 text +
                 ending)
 
+    def offset(self):
+        # TODO: I think this is dependent of import lines. Should not hardcode
+        if self.save_logs:
+            return 0
+        return 1
+
     def iterscribe(self, line, line_num, indentation, program_ast):
         variable_id, variable_type = utils.get_id_and_type(line, program_ast)
         for node in ast.walk(program_ast):
             # TODO: handle nested for loops
             if ('iter' in node._fields and
                 ast.dump(ast.parse(line).body[0]) in ast.dump(node)):
-                self.desugared_lines.insert(node.lineno,
+                line_number = node.lineno - self.offset()
+                self.desugared_lines.insert(line_number,
                                             self.iter_start(node,
                                                             line,
                                                             line_num,
@@ -266,7 +286,8 @@ class Runner(object):
                                                             indentation))
                 iterator_index = "".join(random.choice(string.ascii_uppercase) for _ in range(10))
                 iterator_update = indentation + iterator_index + " += 1\n"
-                self.desugared_lines.insert(node.lineno, indentation[:-4] + iterator_index + " = -1\n")
+                self.desugared_lines.insert(line_number,
+                                            indentation[:-4] + iterator_index + " = -1\n")
                 self.desugared_lines.append(iterator_update)
                 output = ("In iteration ' + str(" +
                           iterator_index +
@@ -283,12 +304,19 @@ class Runner(object):
         identifer, type, and value
         """
         variable_id, variable_type = utils.get_id_and_type(line, program_ast)
-        return (variable_id +
-                " is the ' + " +
-                variable_type +
-                " + ' ' + str(" +
-                variable_id +
-                ")")
+        label = utils.get_label(line, program_ast)
+        output = (variable_id +
+                  " is the ' + " +
+                  variable_type +
+                  " + ' ' + str(" +
+                  variable_id +
+                  ")")
+        if len(self.filtered_labels) > 0:
+            if not label or label not in self.filtered_labels:
+                return ""
+        if label:
+            output += (" + ' (" + label + ")'")
+        return output
 
     def variable_change(self, variable_id, line_num, indentation):
         """A helper method for watch that handles each line that watch
@@ -321,28 +349,30 @@ def process_args():
     parser = argparse.ArgumentParser(description='Let PyScribe make print debugging easier and more efficient.')
     parser.add_argument('python_file', metavar='f', type=python_file_type, nargs='+',
                         help='The Python file with PyScribe API calls.')
-    parser.add_argument('-r', '--run', action='store_true',
-                        help='Run the desugared version')
     parser.add_argument('-d', '--desugared', action='store_true',
                         help='Produce a desugared version of the file with all API calls replaced with valid Python.')
     parser.add_argument('-c', '--clean', action='store_true',
                         help='Produce a clean version of the file with all references to PyScribe removed')
     parser.add_argument('-e', '--extraargs', nargs="+",
-                        help='Arguments intended to be passed to Python file when run. Must be called with --run set')
+                        help='Arguments intended to be passed to Python file when run.')
+    parser.add_argument('-l', '--log', action="store_true",
+                        help='Log the files in a pyscribe_logs.txt file.')
     return parser.parse_args()
 
 def main():
     args = process_args()
-    scribe = Runner()
+    scribe = Runner(args.log)
     program_file = args.python_file[0]
     line_mapping = scribe.gen_line_mapping(program_file)
     program_ast = scribe.gen_ast(program_file)
     desugared_copy = scribe.gen_desugared(line_mapping,
                                           program_file,
                                           program_ast)
+
+    do_run = not args.clean and not args.desugared
     if args.clean:
         scribe.gen_clean_copy(program_file, line_mapping)
-    if args.run:
+    if do_run:
         if args.extraargs:
             subprocess.call(['python', desugared_copy] + [arg for arg in args.extraargs])
         else:
